@@ -12,7 +12,7 @@ module Sequel
       << [] all avg count columns columns! delete each
       empty? fetch_rows first first! get import insert interval last
       map max min multi_insert paged_each range select_hash select_hash_groups select_map select_order_map
-      single_record single_value sum to_hash to_hash_groups truncate update
+      single_record single_record! single_value single_value! sum to_hash to_hash_groups truncate update
     METHS
 
     # Inserts the given argument into the database.  Returns self so it
@@ -203,7 +203,7 @@ module Sequel
     # Calls first.  If first returns nil (signaling that no
     # row matches), raise a Sequel::NoMatchingRow exception.
     def first!(*args, &block)
-      first(*args, &block) || raise(Sequel::NoMatchingRow)
+      first(*args, &block) || raise(Sequel::NoMatchingRow.new(self))
     end
 
     # Return the column value for the first matching record in the dataset.
@@ -268,7 +268,7 @@ module Sequel
     # :commit_every :: Open a new transaction for every given number of records.
     #                  For example, if you provide a value of 50, will commit
     #                  after every 50 records.
-    # :return :: When the :value is :primary_key, returns an array of
+    # :return :: When this is set to :primary_key, returns an array of
     #            autoincremented primary key values for the rows inserted.
     # :server :: Set the server/shard to use for the transaction and insert
     #            queries.
@@ -298,7 +298,7 @@ module Sequel
     #
     # +insert+ handles a number of different argument formats:
     # no arguments or single empty hash :: Uses DEFAULT VALUES
-    # single hash :: Most common format, treats keys as columns an values as values
+    # single hash :: Most common format, treats keys as columns and values as values
     # single array :: Treats entries as values, with no columns
     # two arrays :: Treats first array as columns, second array as values
     # single Dataset :: Treats as an insert based on a selection from the dataset given,
@@ -637,22 +637,51 @@ module Sequel
       _select_map(column, true, &block)
     end
 
-    # Returns the first record in the dataset, or nil if the dataset
-    # has no records. Users should probably use +first+ instead of
-    # this method.
+    # Limits the dataset to one record, and returns the first record in the dataset,
+    # or nil if the dataset has no records. Users should probably use +first+ instead of
+    # this method. Example:
+    #
+    #   DB[:test].single_record # SELECT * FROM test LIMIT 1
+    #   # => {:column_name=>'value'}
     def single_record
-      clone(:limit=>1).each{|r| return r}
-      nil
+      clone(:limit=>1).single_record!
+    end
+
+    # Returns the first record in dataset, without limiting the dataset. Returns nil if
+    # the dataset has no records. Users should probably use +first+ instead of this method.
+    # This should only be used if you know the dataset is already limited to a single record.
+    # This method may be desirable to use for performance reasons, as it does not clone the
+    # receiver. Example:
+    #
+    #   DB[:test].single_record! # SELECT * FROM test
+    #   # => {:column_name=>'value'}
+    def single_record!
+      with_sql_first(select_sql)
     end
 
     # Returns the first value of the first record in the dataset.
     # Returns nil if dataset is empty.  Users should generally use
-    # +get+ instead of this method.
+    # +get+ instead of this method. Example:
+    #
+    #   DB[:test].single_value # SELECT * FROM test LIMIT 1
+    #   # => 'value'
     def single_value
       if r = ungraphed.naked.single_record
-        r.values.first
+        r.each{|_, v| return v}
       end
     end
+
+    # Returns the first value of the first record in the dataset, without limiting the dataset.
+    # Returns nil if the dataset is empty. Users should generally use +get+ instead of this
+    # method.  Should not be used on graphed datasets or datasets that have row_procs that
+    # don't return hashes.  This method may be desirable to use for performance reasons, as
+    # it does not clone the receiver.
+    #
+    #   DB[:test].single_value! # SELECT * FROM test
+    #   # => 'value'
+    def single_value!
+      with_sql_single_value(select_sql)
+    end
     
     # Returns the sum for the given column/expression.
     # Uses a virtual row block if no column is given.
@@ -818,7 +847,7 @@ module Sequel
     # only a single value.  See with_sql_each.
     def with_sql_single_value(sql)
       if r = with_sql_first(sql)
-        r.values.first
+        r.each{|_, v| return v}
       end
     end
 
@@ -915,7 +944,7 @@ module Sequel
       db = @db
       if db.sharded?
         opts = Hash[opts]
-        opts[:server] = @opts[:server] || :read_only
+        opts[:server] = @opts[:server] || (@opts[:lock] ? :default : :read_only)
         opts
       end
       db.execute(sql, opts, &block)
