@@ -468,7 +468,7 @@ module Sequel
       # An alias for calling first! on the model's dataset, but with
       # optimized handling of the single argument case.
       def first!(*args, &block)
-        first(*args, &block) || raise(Sequel::NoMatchingRow)
+        first(*args, &block) || raise(Sequel::NoMatchingRow.new(dataset))
       end
 
       # Clear the setter_methods cache when a module is included, as it
@@ -489,7 +489,7 @@ module Sequel
       #   end
       def inherited(subclass)
         super
-        ivs = subclass.instance_variables.collect{|x| x.to_s}
+        ivs = subclass.instance_variables.collect(&:to_s)
         inherited_instance_variables.each do |iv, dup|
           next if ivs.include?(iv.to_s)
           if (sup_class_value = instance_variable_get(iv)) && dup
@@ -508,15 +508,13 @@ module Sequel
           end
           subclass.instance_variable_set(iv, sup_class_value)
         end
+
         unless ivs.include?("@dataset")
-          if self == Model || !@dataset
-            n = subclass.name
-            unless n.nil? || n.empty?
-              db
-              subclass.set_dataset(subclass.implicit_table_name) rescue nil
-            end
-          elsif @dataset
+          if @dataset && self != Model
             subclass.set_dataset(@dataset.clone, :inherited=>true) rescue nil
+          elsif (n = subclass.name) && !n.to_s.empty?
+            db
+            subclass.set_dataset(subclass.implicit_table_name) rescue nil
           end
         end
       end
@@ -756,7 +754,7 @@ module Sequel
       # dataset methods that accept arguments, you should use define a
       # method directly inside a #dataset_module block.
       def subset(name, *args, &block)
-        dataset_module.subset(name, *args, &block)
+        def_dataset_method(name){filter(*args, &block)}
       end
       
       # Returns name of primary table for the dataset. If the table for the dataset
@@ -787,7 +785,7 @@ module Sequel
 
       # Return the model instance with the primary key, or raise NoMatchingRow if there is no matching record.
       def with_pk!(pk)
-        with_pk(pk) || raise(NoMatchingRow)
+        with_pk(pk) || raise(NoMatchingRow.new(dataset))
       end
   
       # Add model methods that call dataset methods
@@ -851,7 +849,7 @@ module Sequel
         clear_setter_methods_cache
         columns, bad_columns = columns.partition{|x| NORMAL_METHOD_NAME_REGEXP.match(x.to_s)}
         bad_columns.each{|x| def_bad_column_accessor(x)}
-        im = instance_methods.collect{|x| x.to_s}
+        im = instance_methods.collect(&:to_s)
         columns.each do |column|
           meth = "#{column}="
           overridable_methods_module.module_eval("def #{column}; self[:#{column}] end", __FILE__, __LINE__) unless im.include?(column.to_s)
@@ -948,7 +946,7 @@ module Sequel
         if allowed_columns
           allowed_columns.map{|x| "#{x}="}
         else
-          meths = instance_methods.collect{|x| x.to_s}.grep(SETTER_METHOD_REGEXP) - RESTRICTED_SETTER_METHODS
+          meths = instance_methods.collect(&:to_s).grep(SETTER_METHOD_REGEXP) - RESTRICTED_SETTER_METHODS
           meths -= Array(primary_key).map{|x| "#{x}="} if primary_key && restrict_primary_key?
           meths
         end
@@ -1329,9 +1327,10 @@ module Sequel
       def freeze
         values.freeze
         changed_columns.freeze
-        errors
-        validate
-        errors.freeze
+        unless errors.frozen?
+          validate
+          errors.freeze
+        end
         this.freeze if !new? && model.primary_key
         super
       end
@@ -1558,7 +1557,7 @@ module Sequel
       #   a.save_changes # UPDATE artists SET name = 'Bob' WHERE (id = 1)
       #   # => #<Artist {:id=>1, :name=>'Jim', ...}
       def save_changes(opts=OPTS)
-        save(opts.merge(:changed=>true)) || false if modified? 
+        save(Hash[opts].merge!(:changed=>true)) || false if modified? 
       end
   
       # Updates the instance with the supplied values with support for virtual
@@ -1610,7 +1609,7 @@ module Sequel
       #   # Sequel::Error raised
       def set_fields(hash, fields, opts=nil)
         opts = if opts
-          model.default_set_fields_options.merge(opts)
+          Hash[model.default_set_fields_options].merge!(opts)
         else
           model.default_set_fields_options
         end
@@ -1744,6 +1743,32 @@ module Sequel
 
       private
       
+      # Run code directly after the INSERT query, before after_create.
+      # This is only a temporary API, it should not be overridden by external code.
+      def _after_create(pk)
+        @this = nil
+        @new = false
+        @was_new = true
+      end
+
+      # Run code after around_save returns, before calling after_commit.
+      # This is only a temporary API, it should not be overridden by external code.
+      def _after_save(pk)
+        if @was_new
+          @was_new = nil
+          pk ? _save_refresh : changed_columns.clear
+        else
+          @columns_updated = nil
+        end
+        @modified = false
+      end
+
+      # Run code directly after the UPDATE query, before after_update.
+      # This is only a temporary API, it should not be overridden by external code.
+      def _after_update
+        @this = nil
+      end
+
       # Run code before any validation is done, but also run it before saving
       # even if validation is skipped.  This is a private hook.  It exists so that
       # plugins can set values automatically before validation (as the values
@@ -1831,18 +1856,21 @@ module Sequel
   
       # Insert into the given dataset and return the primary key created (if any).
       def _insert_raw(ds)
-        ds.insert(@values)
+        ds.insert(_insert_values)
       end
 
       # Insert into the given dataset and return the hash of column values.
       def _insert_select_raw(ds)
-        ds.insert_select(@values)
+        ds.insert_select(_insert_values)
       end
+
+      # The values hash to use when inserting a new record.
+      alias _insert_values values
       
       # Refresh using a particular dataset, used inside save to make sure the same server
       # is used for reading newly inserted values from the database
       def _refresh(dataset)
-        _refresh_set_values(_refresh_get(dataset) || raise(Error, "Record not found"))
+        _refresh_set_values(_refresh_get(dataset) || raise(NoExistingObject, "Record not found"))
         changed_columns.clear
       end
 
@@ -1861,7 +1889,6 @@ module Sequel
       def _save(opts)
         sh = {:server=>this_server}
         db.after_rollback(sh){after_rollback} if uacr = use_after_commit_rollback
-        was_new = false
         pk = nil
         called_save = false
         called_cu = false
@@ -1869,14 +1896,11 @@ module Sequel
           called_save = true
           raise_hook_failure(:before_save) if before_save == false
           if new?
-            was_new = true
             around_create do
               called_cu = true
               raise_hook_failure(:before_create) if before_create == false
               pk = _insert
-              @this = nil
-              @new = false
-              @was_new = true
+              _after_create(pk)
               after_create
               true
             end
@@ -1899,7 +1923,7 @@ module Sequel
                 changed_columns.reject!{|c| columns.include?(c)}
               end
               _update_columns(@columns_updated)
-              @this = nil
+              _after_update
               after_update
               true
             end
@@ -1909,22 +1933,16 @@ module Sequel
           true
         end
         raise_hook_failure(:around_save) unless called_save
-        if was_new
-          @was_new = nil
-          pk ? _save_refresh : changed_columns.clear
-        else
-          @columns_updated = nil
-        end
-        @modified = false
+        _after_save(pk)
         db.after_commit(sh){after_commit} if uacr
         self
       end
-
+      
       # Refresh the object after saving it, used to get
       # default values of all columns.  Separated from _save so it
       # can be overridden to avoid the refresh.
       def _save_refresh
-        _save_set_values(_refresh_get(this.server?(:default)) || raise(Error, "Record not found"))
+        _save_set_values(_refresh_get(this.server?(:default)) || raise(NoExistingObject, "Record not found"))
         changed_columns.clear
       end
 
@@ -1941,7 +1959,7 @@ module Sequel
       # databases don't like you setting primary key values even
       # to their existing values.
       def _save_update_all_columns_hash
-        v = @values.dup
+        v = Hash[@values]
         Array(primary_key).each{|x| v.delete(x) unless changed_columns.include?(x)}
         v
       end
@@ -2028,7 +2046,7 @@ module Sequel
       
       # If transactions should be used, wrap the yield in a transaction block.
       def checked_transaction(opts=OPTS)
-        use_transaction?(opts) ? db.transaction({:server=>this_server}.merge(opts)){yield} : yield
+        use_transaction?(opts) ? db.transaction({:server=>this_server}.merge!(opts)){yield} : yield
       end
 
       # Change the value of the column to given value, recording the change.
@@ -2066,7 +2084,7 @@ module Sequel
       # Copy constructor -- Duplicate internal data structures.
       def initialize_copy(other)
         super
-        @values = @values.dup
+        @values = Hash[@values]
         @changed_columns = @changed_columns.dup if @changed_columns
         @errors = @errors.dup if @errors
         @this = @this.dup if @this
@@ -2127,14 +2145,14 @@ module Sequel
             set_column_value(m, v)
           elsif strict
             # Avoid using respond_to? or creating symbols from user input
-            if public_methods.map{|s| s.to_s}.include?(m)
-              if Array(model.primary_key).map{|s| s.to_s}.member?(k.to_s) && model.restrict_primary_key?
-                raise Error, "#{k} is a restricted primary key"
+            if public_methods.map(&:to_s).include?(m)
+              if Array(model.primary_key).map(&:to_s).member?(k.to_s) && model.restrict_primary_key?
+                raise MassAssignmentRestriction, "#{k} is a restricted primary key"
               else
-                raise Error, "#{k} is a restricted column"
+                raise MassAssignmentRestriction, "#{k} is a restricted column"
               end
             else
-              raise Error, "method #{m} doesn't exist"
+              raise MassAssignmentRestriction, "method #{m} doesn't exist"
             end
           end
         end
@@ -2157,7 +2175,7 @@ module Sequel
         if type.is_a?(Array)
           type.map{|x| "#{x}="}
         else
-          meths = methods.collect{|x| x.to_s}.grep(SETTER_METHOD_REGEXP) - RESTRICTED_SETTER_METHODS
+          meths = methods.collect(&:to_s).grep(SETTER_METHOD_REGEXP) - RESTRICTED_SETTER_METHODS
           meths -= Array(primary_key).map{|x| "#{x}="} if type != :all && primary_key && model.restrict_primary_key?
           meths
         end
@@ -2238,7 +2256,7 @@ module Sequel
       #   # DELETE FROM artists WHERE (id = 2)
       #   # ...
       def destroy
-        pr = proc{all{|r| r.destroy}.length}
+        pr = proc{all(&:destroy).length}
         model.use_transactions ? @db.transaction(:server=>opts[:server], &pr) : pr.call
       end
 
@@ -2349,7 +2367,7 @@ module Sequel
       # Same as with_pk, but raises NoMatchingRow instead of returning nil if no
       # row matches.
       def with_pk!(pk)
-        with_pk(pk) || raise(NoMatchingRow)
+        with_pk(pk) || raise(NoMatchingRow.new(self))
       end
     end
 
